package com.ruoyi.yh.unit.service.impl;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.k3cloud.constants.K3FormIds;
import com.ruoyi.common.k3cloud.constants.fileds.K3UnitField;
import com.ruoyi.common.k3cloud.service.K3Service;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.yh.unit.service.UnitConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.yh.unit.mapper.UnitMapper;
import com.ruoyi.yh.unit.domain.Unit;
import com.ruoyi.yh.unit.domain.UnitVO;
import com.ruoyi.yh.unit.domain.UnitQuery;
import com.ruoyi.yh.unit.service.UnitService;

/**
 * 单位Service业务层处理
 *
 * @author ruoyi
 * @date 2024-11-18
 */
@Service
@Slf4j
public class UnitServiceImpl implements UnitService
{
    @Autowired
    private UnitMapper unitMapper;

    @Autowired
    private K3Service k3Service;

    @Autowired
    private UnitConverter unitConverter;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private RedisLock redisLock;

    /**
     * 查询单位
     *
     * @param unitId 单位主键
     * @return 单位
     */
    @Override
    public UnitVO selectUnitByUnitId(Long unitId)
    {
        return unitMapper.selectUnitByUnitId(unitId);
    }

    /**
     * 查询单位列表
     *
     * @param unit 单位
     * @return 单位
     */
    @Override
    public List<UnitVO> selectUnitList(UnitQuery unit)
    {
        return unitMapper.selectUnitList(unit);
    }

    /**
     * 新增单位
     *
     * @param unit 单位
     * @return 结果
     */
    @Override
    public int insertUnit(Unit unit)
    {
        return unitMapper.insertUnit(unit);
    }

    /**
     * 修改单位
     *
     * @param unit 单位
     * @return 结果
     */
    @Override
    public int updateUnit(Unit unit)
    {
        return unitMapper.updateUnit(unit);
    }

    /**
     * 批量删除单位
     *
     * @param unitIds 需要删除的单位主键
     * @return 结果
     */
    @Override
    public int deleteUnitByUnitIds(Long[] unitIds)
    {
        return unitMapper.deleteUnitByUnitIds(unitIds);
    }

    /**
     * 删除单位信息
     *
     * @param unitId 单位主键
     * @return 结果
     */
    @Override
    public int deleteUnitByUnitId(Long unitId)
    {
        return unitMapper.deleteUnitByUnitId(unitId);
    }

    @Override
    public void sync() {
        // 查询字段
        scheduledExecutorService.schedule(() -> {
            List<String> fieldKeys = Arrays.asList(
                K3UnitField.F_UNIT_ID,
                K3UnitField.F_NUMBER,
                K3UnitField.F_NAME,
                K3UnitField.F_FORBID_STATUS
            );
            int startRow = 0;
            int limit = 10000;
            int size = 0;

            // 一直查询，直到查到的数据量比limit小，说明查完了
            while(true) {
                // 查询ERP数据
                List<JSONArray> erpList = k3Service.selectList(K3FormIds.UNIT, fieldKeys, startRow, limit);

                // 转为PO
                List<Unit> unitList = unitConverter.toPoByErpList(fieldKeys, erpList);

                // 保存
                for (Unit unit : unitList) {
                    scheduledExecutorService.schedule(() -> {
                        this.saveByErpId(unit);
                    }, 0, TimeUnit.SECONDS);
                }

                size += unitList.size();

                if (unitList.size() < limit) {
                    break;
                } else {
                    startRow += limit;
                }
            }
        }, 0, TimeUnit.SECONDS);
    }

    private int saveByErpId(Unit unit) {
        String errorMsg = "成功";
        try {
            if (unit == null || StringUtils.isBlank(unit.getErpId())) {
                throw new ServiceException("参数错误");
            }
            String lockKey = unit.getErpId();
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.SAVE_UNIT, lockKey), "当前单位正在处理中，请稍后重试");

            try {
                // 若数据库中没有,则新增,否则修改
                UnitVO old = this.selectUnitByErpId(unit.getErpId());
                if (old == null) {
                    return this.insertUnit(unit);
                } else {
                    unit.setUnitId(old.getUnitId());
                    return this.updateUnit(unit);
                }
            } finally {
                redisLock.unlock(RedisLockKey.SAVE_UNIT, lockKey);
            }
        } catch (Exception e) {
            errorMsg = e.getMessage();
            return 0;
        } finally {
            // TODO 记录日志
            log.info("保存单位:{}",  errorMsg);
        }
    }

    private UnitVO selectUnitByErpId(String erpId) {
        if (StringUtils.isBlank(erpId)) {
            return null;
        }
        UnitQuery query = new UnitQuery();
        query.setErpId(erpId);
        return this.selectOne(query);
    }

    private UnitVO selectOne(UnitQuery query) {
        PageHelper.startPage(1, 1);
        List<UnitVO> list = this.selectUnitList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }
}
